#coding:utf-8
'''
Created on 06.09.2011

@author: akvarats
'''

import json

from django.test import TestCase

from m3_replica import (JSONDataSource, SimpleExchange, CallbackDataTarget, ) 

from models import SimpleModel


class SourcesTests(TestCase):
    '''
    Тесты для источников данных
    '''
    def test1(self):
        
        initial_data = [{'a1': '2', 'a3': '4',}, 
                        {'a5': '6', 'a7': '8',},
                        {'a9': '10', 'a11': '12',},]
        
        json_data = json.JSONEncoder().encode(initial_data)
        
        source = JSONDataSource(json_data=json_data)
        
        source.open_session()
        
        i = 0
        for row in source.read():
            
            if i == 0:
                self.assertEqual(row['a1'], '2')
                self.assertEqual(row['a3'], '4')
            elif i == 1:
                self.assertEqual(row['a5'], '6')
                self.assertEqual(row['a7'], '8')
            elif i == 2:
                self.assertEqual(row['a9'], '10')
                self.assertEqual(row['a11'], '12')
            else:
                self.fail(u'Прочитано слишком много строк из источника данных.')
            
            i += 1
            
        self.assertEquals(i, 3)
      
        
class TargetTests(TestCase):
    '''
    Тесты для классов приемщиков данных
    '''
    
    imported_objects = []
    
    def test_callback_target(self):
        
        def writer(objects):
            '''
            '''
            if isinstance(objects, list):
                TargetTests.imported_objects.extend(objects)
            else:
                TargetTests.imported_objects.append(objects)
       
        external_data = [{'ext_id': '1', 'ext_code': '2', 'ext_name': '3'}, 
                        {'ext_id': '4', 'ext_code': '5', 'ext_name': '6'},
                        {'ext_id': '7', 'ext_code': '8', 'ext_name': '9'},]
        json_data = json.JSONEncoder().encode(external_data)
        
        exchange = SimpleExchange(source=JSONDataSource(json_data = json_data),
                                  target=CallbackDataTarget(write_callback=writer),
                                  model=SimpleModel, 
                                  field_map = {'code': 'ext_code',
                                               'name': 'ext_name',})
        
        TargetTests.imported_objects = []
        
        exchange.run()
        
        self.assertEqual(len(TargetTests.imported_objects), len(external_data))
        
        for i in xrange(0, len(external_data)):
            self.assertTrue(isinstance(TargetTests.imported_objects[i], SimpleModel))
            
            self.assertEqual(TargetTests.imported_objects[i].code, external_data[i]['ext_code'])
            self.assertEqual(TargetTests.imported_objects[i].name, external_data[i]['ext_name'])
            
            
            